<?php
namespace App\Factory;

use App\Exception;

class BranchFactory {

    private $connection;

    public function __construct (\Doctrine\DBAL\Connection $connection)
    {
        $this->connection = $connection;
    }
    
    public function fetchBranch($merchantId)
    {
        
        if(empty($merchantId)) {
            throw new Exception\ParamsException("Não foi possivel integrar o pedido, merchantId (Shoppub) inválido e/ou vazio.");
        }
        
        $sql = "
            SELECT CDFILIAL 
              
              FROM INTEGRACAOFILI IF 
              
              JOIN INTEGRACAO I
                ON IF.CDINTEGRACAO = I.CDINTEGRACAO
            
             WHERE I.IDINTEGRACAO  = :IDINTEGRACAO
               AND IF.IDTOKEN      = :CDFILIALEXT
        ";

        $params = [
            ":IDINTEGRACAO" => "IFO",
            ":CDFILIALEXT"  => $merchantId
        ];
        
        $branchInfo = $this->connection->fetchAssoc($sql, $params);
        
        if(empty($branchInfo['CDFILIAL'])) {
            throw new Exception\ParamsException("
                Não existe uma filial associada ao {$merchantId}.<br>
                Para associar uma filial a este merchantID, verifique a seção 'Configurando o Token das Filiais', no manual da integração.");
        }

        return $branchInfo['CDFILIAL'];
    }
    
    public function getStandardVendor($branch)
    {
        $sql = "
            SELECT CDVENDEDOR
              FROM PARAVEND
             WHERE CDFILIAL = :CDFILIAL";
            
        $params = [
            ":CDFILIAL" => $branch    
        ];
        
        $vendor = $this->connection->fetchAll($sql, $params);
        
        if(empty($vendor)) 
            throw new Exception\ParamsException("
                Não existe um vendedor padrão atrelado a filial {$branch}.");
        
        if(!isset($vendor[0]["CDVENDEDOR"])) 
            throw new Exception\ParamsException("
                Não existe um vendedor padrão atrelado a filial {$branch}.");
        
        return $vendor[0]["CDVENDEDOR"];
    }
    
    public function getStandardStore($branch, $vendor)
    {
        $sql = "
            SELECT C.CDLOJA 
              FROM VENDEDOR V
              
              JOIN CAIXA C
                ON C.CDCAIXA  = V.CDCAIXA
              
              JOIN LOJA L 
                ON C.CDLOJA   = L.CDLOJA
               AND C.CDFILIAL = L.CDFILIAL
            
             WHERE V.CDVENDEDOR = :CDVENDEDOR
               AND C.CDFILIAL   = :CDFILIAL        
        ";
            
        $params = array(
            ":CDFILIAL"   => $branch,
            ":CDVENDEDOR" => $vendor
        );
        
        $store = $this->connection->fetchAll($sql, $params);
        
        if(empty($store))
            throw new Exception\ParamsException("
                Não existe uma loja associada a filial {$branch} e vendedor padrão código {$vendor}.<br>
                Caso necessite de ajuda com esta parametrização, entre em contato com a equipe de suporte Teknisa.");
        
        if(!isset($store[0]["CDLOJA"])) 
            throw new Exception\ParamsException("
                Não existe uma loja associada a filial {$branch} e vendedor padrão código {$vendor}.<br>
                Caso necessite de ajuda com esta parametrização, entre em contato com a equipe de suporte Teknisa.");
        
        return $store[0]["CDLOJA"];
    }
    
    public function getStandardCashier($branch, $vendor)
    {
        $sql = "
            SELECT CDCAIXA, CDOPERADOR
              FROM VENDEDOR
             WHERE CDFILIAL   = :CDFILIAL
               AND CDVENDEDOR = :CDVENDEDOR";
            
        $params = array(
            ":CDFILIAL"   => $branch,
            ":CDVENDEDOR" => $vendor
        );
        
        $cashier = $this->connection->fetchAll($sql, $params);
        
        if(empty($cashier)) 
            throw new Exception\ParamsException("
                Não existe um caixa associado a filial {$branch} e vendedor padrão código {$vendor}.<br>
                Caso necessite de ajuda com esta parametrização, entre em contato com a equipe de suporte Teknisa.");
        
        if(!isset($cashier[0]["CDCAIXA"])) 
            throw new Exception\ParamsException("
                Não existe um caixa associado a filial {$branch} e vendedor padrão código {$vendor}.<br>
                Caso necessite de ajuda com esta parametrização, entre em contato com a equipe de suporte Teknisa.");
        
        return $cashier[0];
    }
    
    public function getStandardClient($branch)
    {
        $sql = "
            SELECT CDCLIENTE
              FROM PARAVEND
            WHERE CDFILIAL = :CDFILIAL";
            
        $params = array(
            ":CDFILIAL" => $branch
        );
        
        $stdClient = $this->connection->fetchAll($sql, $params)[0];
        
        if(empty($stdClient["CDCLIENTE"])) 
            throw new Exception\ParamsException("
                Não existe um cliente padrão associado a filial {$branch}.<br>
                Para associar o cliente padrão para esta filial, verifique a seção 'Parametrização do Delivery', no manual da integração.");
        
        return $stdClient;
    }

    public function getStandardConsumerType($branch)
    {
        $sql = "
            SELECT CDTIPOCONS, CDCCUSCLIE
              FROM PARAVEND
            WHERE CDFILIAL = :CDFILIAL";
            
        $params = array(
            ":CDFILIAL" => $branch    
        );
        
        $consumerT = $this->connection->fetchAll($sql, $params)[0];
        
        if(empty($consumerT["CDTIPOCONS"]))
            throw new Exception\ParamsException("
                Não existe um tipo de consumidor padrão associado a filial {$branch}.<br>
                Para associar o tipo de consumidor para esta filial, verifique a seção 'Parametrização do Delivery', no manual da integração.");

        if(empty($consumerT["CDCCUSCLIE"]))
            throw new Exception\ParamsException("
                Não existe um centro de custo associado a filial {$branch}.<br>
                Para associar o centro de custo para esta filial, verifique a seção 'Parametrização do Delivery', no manual da integração.");
        
        return $consumerT;
    }

    public function getProductControlId($branch, $store)
    {
        $sql = "
            SELECT IDCONTROPROD
              FROM LOJA
            WHERE CDFILIAL = :CDFILIAL
              AND CDLOJA = :CDLOJA";
              
        $params = array(
            ":CDFILIAL" => $branch,
            ":CDLOJA"   => $store
        );
        
        $idControlProd = $this->connection->fetchAll($sql, $params);
        
        if (empty($idControlProd)) 
            throw new Exception\ParamsException("
                A tela de controle de produção não está parametrizada para a filial {$branch}.<br>
                Caso necessite de ajuda com esta parametrização, entre em contato com a equipe de suporte Teknisa.");
        
        if (!isset($idControlProd[0]["IDCONTROPROD"]))
            throw new Exception\ParamsException("
                A tela de controle de produção não está parametrizada para a filial {$branch}.<br>
                Caso necessite de ajuda com esta parametrização, entre em contato com a equipe de suporte Teknisa.");
        
        return $idControlProd[0]["IDCONTROPROD"];
    }
    
    public function getBranchCityCode($branch)
    {
        $sql = "
            SELECT CDMUNICIPIO
              FROM ENDEFILI
             WHERE CDFILIAL = :CDFILIAL";
             
        $params = array(
            ":CDFILIAL" => $branch
        );
        
        return $this->connection->fetchAll($sql, $params)[0]["CDMUNICIPIO"];
    }
    
    public function getBranchState($branch)
    {
        $sql = "
            SELECT SGESTADO
              FROM ENDEFILI
             WHERE CDFILIAL = :CDFILIAL
        ";
        
        $params = array(
            ":CDFILIAL" => $branch
        );
        
        return $this->connection->fetchAll($sql, $params)[0]["SGESTADO"];
    }
    
    public function getDeliveryAreas()
    {
        $sql = " 
            SELECT  A.NMAREA ||' - ' || A.DSAREA as DSAREAATEND,
                    REPLACE(A.COORDAREA, :DOUBLE, :SINGLE) COORDAREA,
                    FA.NRORDATEND
              
              FROM AREAATEND A
             
              JOIN FILIALAREA FA
                ON A.CDAREA = FA.CDAREA
            
              JOIN PARAVEND PA
                ON FA.CDFILIAL = PA.CDFILIAL
        ";
    
        $params = [
			'SINGLE' => "'",
			'DOUBLE' => '"'
		];
		
		try {
		    $deliveryAreas = $this->connection->fetchAll($sql, $params);
		    
		    return [
		        'error' => false,
		        'areas' => $deliveryAreas
	        ];
		} catch (\Exception $e) {
		    return [
                'error' => true,
                'message' => 'Erro ao buscar areas de atendimento. Erro: '.$e->getMessage()
	        ];
		}
    }
    
    public function isPosEnabledToAutomaticOrderPrinting ($branch, $store, $cashier)
    {
        $sql = "
            SELECT IDIMPAUTCUPEXP, IDIMPAUTCUPFOS
              FROM CAIXA
             WHERE CDFILIAL = :CDFILIAL
               AND CDLOJA   = :CDLOJA
               AND CDCAIXA  = :CDCAIXA";
               
        $params = [
             ":CDFILIAL" => $branch,
             ":CDLOJA"   => $store,
             ":CDCAIXA"  => $cashier
        ]; 
        
        $isAutomaticPrint = $this->connection->fetchAssoc($sql, $params);
        if ((empty($isAutomaticPrint["IDIMPAUTCUPEXP"])) && (empty($isAutomaticPrint["IDIMPAUTCUPFOS"]))) {
            return null;
        }
            
        return $isAutomaticPrint;
    }
}
